% Example  9.14 - Alternative MATLAB code suggested by Tom Lane
% of The MathWorks, Inc.
% This contains the complete steps for
% classification trees - cross-validation

% We will use the iris data - partition into 5 groups
% We will not randomly sample into groups, but will
% keep the class fraction the same.

% Class 1: iris setosa
% Class 2: iris versicolor
% Class 3: iris virginica

load iris
% Attach class labels.
setosa(:,5)=1;
versicolor(:,5)=2;
virginica(:,5)=3;
X = [setosa;versicolor;virginica];
n = 150;	% total number of data points
% Set up some cell arrays to hold the data.
ind = cell(1,5);
train = cell(1,5);
test = cell(1,5);
treek = cell(1,5);
treeseqk = cell(1,5);
for i = 1:5
    ind{i} = i:5:50;
    % Set up the testing set.
    test{i} = [setosa(ind{i},:);versicolor(ind{i},:);virginica(ind{i},:)];
    % Set up the training set. Remove the points in the test set.
	tmp1 = setosa;
    tmp1(ind{i},:) = [];
	tmp2 = versicolor;
    tmp2(ind{i},:) = [];
	tmp3 = virginica;
    tmp3(ind{i},:) = [];
    train{i} = [tmp1;tmp2;tmp3];
end
% Grow all of the trees.
pies = ones(1,3)/3;
maxn = 2;	% get large trees
clas = 1:3;
Nk = [50,50,50];
tree = csgrowc(X,maxn,clas,Nk,pies);
Nk1 = [40 40 40];
for i = 1:5
    treek{i} = csgrowc(train{i},maxn,clas,Nk1,pies);
end
% Now prune each sequence.
treeseq = csprunec(tree);
for i = 1:5
    treeseqk{i} = csprunec(treek{i});
end

K=length(treeseq);
alpha = zeros(1,K);
% Find the sequence of alphas.
for i = 1:K
	alpha(i) = treeseq{i}.alpha;
end
alphak = cell(1,5);
for i = 1:5
    Kk = length(treeseqk{i});
    alphatmp = zeros(1,Kk);
    for j = 1:Kk
        alphatmp(j) = treeseqk{i}{j}.alpha;
    end
    alphak{i} = alphatmp;
end

% Get the akprime equivalent values for the main tree.
for i = 1:K-1
	akprime(i) = sqrt(alpha(i)*alpha(i+1));
end


% Now find the error for each tree according to the value of ak'  
n=150;
k = length(akprime);
Rk = zeros(1,k);
seRk = zeros(1,k);

for i = 1:k
	misclass = zeros(1,n);
    for j = 1:5
        % Find the corresponding tree from each partition.
        ind = find(alphak{j} <= akprime(i));
        % Get the tree that corresponds to that one.
        tk = treeseqk{j}{ind(end)};
        % Get the misclassified points in the test set.
        for jj = 1:30   % loop through the points
            [c,pclass,node] = cstreec(test{j}(jj,1:4),tk);
            if c ~= test{j}(jj,5)
                misclass((j-1)*30+jj) = 1;
            end
        end
    end
           
    Rk(i) = mean(misclass);
	seRk(i) = ((n-1)/n)*var(misclass);
end

% Get the standard deviation for the smallest value.
sehat = sqrt(seRk/n);
[Rkmin,mi] = min(Rk);
tol = Rkmin+sehat(mi);
	
	
